﻿namespace SampleServer
{
    using System;
    using System.Net;
    using System.Windows.Forms;
    using Tenma.HomeMatic.Remoting;

    public partial class MainForm : Form
    {
        private delegate void LogDelegate(string address, string valueKey, object value);

        private const int ServicePort = 54321;

        private IRpcProxy proxy;
        private RpcService service;
        private LogDelegate logDelegate;

        public MainForm()
        {
            this.InitializeComponent();
            this.logDelegate = new LogDelegate(this.LogEvent);

            this.service = new RpcService(MainForm.ServicePort, new ServiceHandler(this));
            this.service.Start();

            this.proxy = null;
        }

        public void BeginLogEvent(string address, string valueKey, object value)
        {
            this.BeginInvoke(this.logDelegate, new object[] { address, valueKey, value });
        }

        private void LogEvent(string address, string valueKey, object value)
        {
            ListViewItem item = new ListViewItem();
            item.Text = DateTime.Now.ToString();

            item.SubItems.Add(address);
            item.SubItems.Add(valueKey);
            item.SubItems.Add(value.ToString());

            this.listView.Items.Add(item);
        }

        private void Form1_FormClosed(object sender, FormClosedEventArgs e)
        {
            if (this.proxy != null)
            {
                this.proxy.Shutdown(this.GetServiceUrl());
            }

            this.service.Shutdown();
        }

        private void connectButton_Click(object sender, EventArgs e)
        {
            this.connectButton.Enabled = false;
            this.urlTextBox.ReadOnly = true;

            this.proxy = RpcProxyFactory.Create(this.urlTextBox.Text);
            this.proxy.Init(this.GetServiceUrl(), "id");
        }

        private string GetServiceUrl()
        {
            return string.Format("http://{0}:{1}/",
                this.GetIp(),
                MainForm.ServicePort);
        }

        private string GetIp()
        {
            return Dns.GetHostEntry(Dns.GetHostName()).AddressList[0].ToString();
        }
    }
}
