﻿namespace Tenma.HomeMatic.Remoting
{
    /// <summary>
    /// Definiert die Gerätebeschreibung.
    /// </summary>
    public interface IDeviceDescription
    {
        /// <summary>
        /// Liefert die Seriennummer des Geräts.
        /// </summary>
        string Address { get; }

        /// <summary>
        /// Liefert den Typ des Geräts.
        /// </summary>
        string DeviceType {get; }

        /// <summary>
        /// Ermittelt, ob des sich um ein Gerät handelt.
        /// </summary>
        bool IsDevice { get; }

        /// <summary>
        /// Ermittelt, ob es sich um einen Kanal handelt.
        /// </summary>
        bool IsChannel { get; }

        /// <summary>
        /// Liefert die Kanäle eines Geräts.
        /// </summary>
        string[] Children { get; }

        /// <summary>
        /// Liefert das übergeordnete Gerät eines Kanals.
        /// </summary>
        string Parent { get; }

        /// <summary>
        /// Liefert den Typ des übergeordneten Geräts eines Kanals.
        /// </summary>
        string ParentType { get; }

        /// <summary>
        /// Liefert den Index des Kanals.
        /// </summary>
        int Index { get; }

        /// <summary>
        /// Ermittelt, ob für ein logisches Gerät AES aktiv ist.
        /// </summary>
        int IsAesActive { get; }

        /// <summary>
        /// Liefert die Paramsets, die für ein logisches Gerät 
        /// verfügbar sind.
        /// </summary>
        string[] Paramsets { get; }

        /// <summary>
        /// Liefert die Firmware-Version des Geräts.
        /// </summary>
        string FirmwareVersion { get; }

        /// <summary>
        /// Liefert die verfügbare Firmware-Version eines Geräts.
        /// </summary>
        string AvailableFirmwareVersion { get; }

        /// <summary>
        /// Liefert die Version der Gerätebeschreibung.
        /// </summary>
        int Version { get; }

        /// <summary>
        /// Liefert die Flags des Geräts.
        /// </summary>
        int Flags { get; }

        /// <summary>
        /// Liefert die Rollen, die der Kanals als Sender
        /// einnehmen kann.
        /// </summary>
        string LinkSourceRoles { get; }

        /// <summary>
        /// Liefert die Rollen, die der Kanal als Empfänger
        /// einnehmen kann.
        /// </summary>
        string LinkTargetRoles { get; }

        /// <summary>
        /// Ermittelt, ob der Kanal ein Sender oder
        /// ein Empfänger ist.
        /// </summary>
        int Direction { get; }

        /// <summary>
        /// Liefert die Adresse des Partners in der
        /// Kanalgruppe.
        /// </summary>
        string Group { get; }

        /// <summary>
        /// Liefert die Adresse der virtuellen Team-Kanals.
        /// </summary>
        string Team { get; }

        /// <summary>
        /// Liefert das Tag für Team-Kanäle.
        /// </summary>
        string TeamTag { get; }

        /// <summary>
        /// Liefert die Kanäle eines Teamgeräts.
        /// </summary>
        string[] TeamChannels { get; }

        /// <summary>
        /// Liefert die Seriennummer des zugeordneten 
        /// BidCoS-Interfaces.
        /// </summary>
        string InterfaceAddress { get; }

        /// <summary>
        /// Ermittelt, ob das Roaming-Feature aktiv ist.
        /// </summary>
        int UseRoaming { get; }
    }
}
