﻿namespace Tenma.HomeMatic.Remoting
{
    using System;
    using System.Collections.Generic;

    /// <summary>
    /// Definiert die Beschreibung eines Parameters.
    /// </summary>
    public interface IParameterDescription
    {
        /// <summary>
        /// Liefert den Typ des Parameters.
        /// </summary>
        string ParameterType { get; }

        /// <summary>
        /// Liefert die Operationen des Parameters.
        /// </summary>
        int Operations { get; }

        /// <summary>
        /// Liefert die Flags des Parameters.
        /// </summary>
        int Flags { get; }

        /// <summary>
        /// Liefert den Standardwert des Parameters.
        /// </summary>
        object DefaultValue { get; }

        /// <summary>
        /// Liefert den minimalen Wert des Parameters.
        /// </summary>
        object MinValue { get; }

        /// <summary>
        /// Liefert den maximalen Wert des Parameters.
        /// </summary>
        object MaxValue { get; }

        /// <summary>
        /// Liefert die Einheit des Parameters.
        /// </summary>
        string Unit { get; }

        /// <summary>
        /// Liefert die Folgenummer des Parameters
        /// für die Anzeige.
        /// </summary>
        int TabOrder { get; }

        /// <summary>
        /// Liefert das User-Control für die Anzeige.
        /// </summary>
        string Control { get; }

        /// <summary>
        /// Liefert spezielle Werte des Parameters.
        /// </summary>
        Dictionary<string, object> SpecialValues { get; }

        /// <summary>
        /// Liefert die Liste der erlaubten Werte des
        /// Parameters.
        /// </summary>
        String[] ValueList { get; }
    }
}
