﻿namespace Tenma.HomeMatic.Remoting
{
    using System;
    using CookComputing.XmlRpc;

    /// <summary>
    /// Ausnahme für Clientanfragen.
    /// </summary>
    public class RpcException : Exception
    {
        /// <summary>
        /// Fehlercode der Ausnahme.
        /// </summary>
        private int errorCode;

        /// <summary>
        /// Erstellt und initialisiert eine neue Ausnahme.
        /// </summary>
        /// <param name="ex">Innere Ausnahme</param>
        public RpcException(Exception ex)
            : base("RpcException", ex)
        {
            if (ex is XmlRpcFaultException)
            {
                this.errorCode = ((XmlRpcFaultException)ex).FaultCode;
            }
            else
            {
                this.errorCode = 0;
            }
        }

        /// <summary>
        /// Liefert den Fehlercode.
        /// </summary>
        public int ErrorCode
        {
            get
            {
                return this.errorCode;
            }
        }

        /// <summary>
        /// Liefert eine kurze Fehlerbeschreibung.
        /// </summary>
        public string ErrorMessage
        {
            get
            {
                switch (this.errorCode)
                {
                    case -1:
                        return "general error";
                    case -2:
                        return "unknown device or channel";
                    case -3:
                        return "unknown paramset";
                    case -4:
                        return "device address expected";
                    case -5:
                        return "unknown parameter or value";
                    case -6:
                        return "unsupported option";
                    default: 
                        return this.InnerException.ToString();
                }
            }
        }

        /// <summary>
        /// Liefert die Zeichenkettendarstellung der Ausnahme.
        /// </summary>
        /// <returns>Zeichenkettendarstellung der Ausnahme</returns>
        public override string ToString()
        {
            return string.Format("RpcException {0}: {1}", this.errorCode, this.ErrorMessage);
        }
    }
}
