﻿namespace Tenma.HomeMatic.Remoting
{
    using System;
    using System.Collections;
    using System.Collections.Generic;
    using System.Text;

    using Nwc.XmlRpc;

    [XmlRpcExposed]
    public class RpcService
    {
        private IRpcServiceHandler serviceHandler;
        private XmlRpcServer server;

        public RpcService(int port, IRpcServiceHandler serviceHandler)
        {
            this.server = new XmlRpcServer(port);
            this.serviceHandler = serviceHandler;

            this.server.Add(string.Empty, this);
        }

        public IRpcServiceHandler ServiceHandler
        {
            get
            {
                return this.serviceHandler;
            }

            set
            {
                this.serviceHandler = value;
            }
        }

        public void Start()
        {
            this.server.Start();
        }

        public void Shutdown()
        {
            this.server.Stop();
        }

        [XmlRpcExposed]
        public void Event(string interfaceId, string address, string valueKey, object value)
        {
            if (this.serviceHandler != null)
            {
                this.serviceHandler.Event(interfaceId, address, valueKey, value);
            }
        }

        [XmlRpcExposed]
        public IList<IDeviceDescription> listDevices(string interfaceId)
        {
            if (this.serviceHandler != null)
            {
                return this.serviceHandler.ListDevices(interfaceId);
            }
            else
            {
                return new List<IDeviceDescription>();
            }
        }

        [XmlRpcExposed]
        void newDevices(string interfaceId, IList devices)
        {
            if (this.serviceHandler != null)
            {
                this.serviceHandler.NewDevices(interfaceId, new IDeviceDescription[] { });
            }
        }

        [XmlRpcExposed]
        void deleteDevices(string interfaceId, IList devices)
        {
            if (this.serviceHandler != null)
            {
                string[] addresses = new string[devices.Count];
                for (int i = 0; i < devices.Count; i++)
                {
                    addresses[i] = devices[i].ToString();
                }

                this.serviceHandler.DeleteDevices(interfaceId, addresses);
            }
        }

        [XmlRpcExposed]
        void updateDevice(string interfaceId, string address, int hint)
        {
            if (this.serviceHandler != null)
            {
                this.serviceHandler.UpdateDevice(interfaceId, address, hint);
            }
        }

    }
}
