﻿namespace Tenma.HomeMatic.Remoting
{
    /// <summary>
    /// Kapselt RSSI-Informationen.
    /// </summary>
    public class RssiInfo
    {
        /// <summary>
        /// Ungültiger RSSI-Wert.
        /// </summary>
        public const int InvalidRssiValue = 65536;

        /// <summary>
        /// Seriennummer des Senders.
        /// </summary>
        private string sender;

        /// <summary>
        /// Seriennummer des Empfängers.
        /// </summary>
        private string receiver;

        /// <summary>
        /// RSSI Wert.
        /// </summary>
        private int rssiValue;

        /// <summary>
        /// Erstellt und initialisiert RSSI-Informationen.
        /// </summary>
        public RssiInfo()
            : this (string.Empty, string.Empty, RssiInfo.InvalidRssiValue)
        {
        }

        /// <summary>
        /// Erstellt und initialisiert RSSI-Informationen.
        /// </summary>
        /// <param name="sender">Seriennumemr des Senders</param>
        /// <param name="receiver">Seriennummer des Empfängers</param>
        /// <param name="rssiInfo">RSSI Wert</param>
        public RssiInfo(string sender, string receiver, int rssiInfo)
        {
            this.sender = sender;
            this.receiver = receiver;
            this.rssiValue = rssiInfo;
        }

        /// <summary>
        /// Ermittelt, ob der RSSI-Wert gültig ist.
        /// </summary>
        public bool IsValid
        {
            get
            {
                return (this.rssiValue != RssiInfo.InvalidRssiValue);
            }
        }

        /// <summary>
        /// Liefert die Seriennummer des Senders.
        /// </summary>
        public string Sender
        {
            get
            {
                return this.sender;
            }
        }

        /// <summary>
        /// Liefert die Seriennummer des Empfängers.
        /// </summary>
        public string Receiver
        {
            get
            {
                return this.receiver;
            }
        }

        /// <summary>
        /// Liefert den RSSI Wert.
        /// </summary>
        public int RssiValue
        {
            get
            {
                return this.rssiValue;
            }
        }
    }
}
