﻿namespace Tenma.HomeMatic.Remoting
{
    using System;
    using System.Collections.Generic;
    using System.Text;
    using System.Net;

    /// <summary>
    /// Proxy für die Ausführung von HomeMatic Script
    /// auf der HomeMatic Zentrale
    /// </summary>
    public class ScriptProxy
    {
        /// <summary>
        /// Remote HomeMatic Script Port
        /// auf der HomeMatic Zentrale
        /// </summary>
        public const int Port = 8181;

        /// <summary>
        /// WebClient für die Kommunikation
        /// </summary>
        private WebClient webClient;

        /// <summary>
        /// IP-Adresse des Proxys
        /// </summary>
        private IPAddress ipAddress;

        /// <summary>
        /// Erstellt und initialisiert
        /// einen neuen Proxy
        /// </summary>
        public ScriptProxy()
            : this(IPAddress.Loopback)
        {
        }

        /// <summary>
        /// Erstellt und initialisiert einen
        /// neuen Proxy
        /// </summary>
        /// <param name="address">IP-Adresse des Proxys</param>
        public ScriptProxy(IPAddress address)
        {
            this.ipAddress = address;
            this.webClient = new WebClient();
        }

        /// <summary>
        /// Liefert den WebClient für die Kommunikation.
        /// </summary>
        public WebClient WebClient
        {
            get
            {
                return this.webClient;
            }
        }

        /// <summary>
        /// Liefert oder setzt die IP-Adresse
        /// des Proxys.
        /// </summary>
        public IPAddress IPAddress
        {
            get
            {
                return this.ipAddress;
            }

            set
            {
                this.ipAddress = value;
            }
        }

        /// <summary>
        /// Liefert die URL des Proxys.
        /// </summary>
        public string Url
        {
            get
            {
                return string.Format("http://{0}:{1}/Test.exe", this.ipAddress, ScriptProxy.Port);
            }
        }

        /// <summary>
        /// Führt ein HomeMatic Script aus.
        /// </summary>
        /// <param name="script">Auszuführendes Script</param>
        /// <returns>Rückgabe des Aufrufs</returns>
        public IScriptResponse Invoke(string script)
        {
            return new ScriptResponse(this.webClient.UploadString(this.Url, "POST", script));
        }
    }
}
