﻿namespace Tenma.HomeMatic.Remoting
{
    using System;
    using System.Collections.Generic;
    using System.Text;
    using System.Xml;

    /// <summary>
    /// Antwort eines ausgeführten HomeMatic Scripts.
    /// </summary>
    internal class ScriptResponse : Dictionary<string, string>, IScriptResponse
    {
        /// <summary>
        /// ID der Standardausgabe.
        /// </summary>
        public const string StdoutId = "_STDOUT_";

        /// <summary>
        /// ID der kompletten Ausgabe des Scripts.
        /// </summary>
        public const string ResponseTextId = "_RESPONSE_TEXT_";

        /// <summary>
        /// Erstellt und initialisiert eine neue 
        /// Antwort.
        /// </summary>
        /// <param name="responseText">Text der Antwort</param>
        public ScriptResponse(string responseText)
            : base()
        {
            this[ScriptResponse.ResponseTextId] = responseText;

            int xmlIndex = responseText.IndexOf("<xml>");
            if (xmlIndex >= 0)
            {
                this[ScriptResponse.StdoutId] = responseText.Substring(0, xmlIndex);
                this.ParseVariables(responseText.Substring(xmlIndex));
            }
            else
            {
                this[ScriptResponse.StdoutId] = string.Empty;
            }
        }

        /// <summary>
        /// Liefert den kompletten Text der Antwort.
        /// </summary>
        public string ResponseText
        {
            get
            {
                return this[ScriptResponse.ResponseTextId];
            }
        }

        /// <summary>
        /// Liefert die Standardausgabe der Antwort.
        /// </summary>
        public string Stdout
        {
            get
            {
                return this[ScriptResponse.StdoutId];
            }
        }

        /// <summary>
        /// Parst die Variablen der Antwort.
        /// </summary>
        /// <param name="variables">XML-Teil der Antwort</param>
        private void ParseVariables(string variables)
        {
            try
            {
                XmlDocument doc = new XmlDocument();
                doc.LoadXml(variables);
                foreach (XmlNode node in doc.DocumentElement.ChildNodes)
                {
                    if (node.NodeType == XmlNodeType.Element)
                    {
                        this[node.Name] = node.InnerText;
                    }
                }
            }
            catch
            {
                // leer
            }
        }
    }
}
