﻿namespace Tenma.HomeMatic.Remoting.XmlRpc.Net
{
    using System;
    using System.Collections.Generic;
    using System.Text;

    using CookComputing.XmlRpc;

    public class DeviceDescription: IDeviceDescription
    {
        private string address;
        private string deviceType;
        private string[] children;
        private string parent;
        private string parentType;
        private int index;
        private int isAesActive;
        private string[] paramsets;
        private string firmwareVersion;
        private string availableFirmwareVersion;
        private int version;
        private int flags;
        private string linkSourceRoles;
        private string linkTargetRoles;
        private int direction;
        private string group;
        private string team;
        private string teamTag;
        private string[] teamChannels;
        private string interfaceAddress;
        private int useRoaming;
        
        public DeviceDescription()
        {
            this.address = string.Empty;
            this.deviceType = string.Empty;
            this.children = new string[] { };
            this.parent = string.Empty;
            this.parentType = string.Empty;
            this.index = -1;
            this.isAesActive = 0;
            this.paramsets = new string[] { };
            this.firmwareVersion = string.Empty;
            this.availableFirmwareVersion = string.Empty;
            this.version = -1;
            this.flags = 0;
            this.linkSourceRoles = string.Empty;
            this.linkTargetRoles = string.Empty;
            this.direction = 0;
            this.group = string.Empty;
            this.team = string.Empty;
            this.teamTag = string.Empty;
            this.teamChannels = new string[] { };
            this.interfaceAddress = string.Empty;
            this.useRoaming = 0;
        }

        [XmlRpcMissingMapping(MappingAction.Ignore)]
        public bool IsDevice
        {
            get
            {
                return string.IsNullOrEmpty(this.parent);
            }
        }

        [XmlRpcMissingMapping(MappingAction.Ignore)]
        public bool IsChannel
        {
            get
            {
                return !this.IsDevice;
            }
        }

        [XmlRpcMember("ADDRESS")]
        public string Address
        {
            get
            {
                return this.address;
            }

            set
            {
                this.address = value;
            }
        }

        [XmlRpcMember("TYPE")]
        public string DeviceType
        {
            get
            {
                return this.deviceType;
            }

            set
            {
                this.deviceType = value;
            }
        }

        [XmlRpcMember("CHILDREN")]
        [XmlRpcMissingMapping(MappingAction.Ignore)]
        public string[] Children
        {
            get
            {
                return this.children;
            }

            set
            {
                this.children = value;
            }
        }

        [XmlRpcMember("PARENT")]
        public string Parent
        {
            get
            {
                return this.parent;
            }

            set
            {
                this.parent = value;
            }
        }

        [XmlRpcMember("PARENT_TYPE")]
        [XmlRpcMissingMapping(MappingAction.Ignore)]
        public string ParentType
        {
            get
            {
                return this.parentType;
            }

            set
            {
                this.parentType = value;
            }
        }

        [XmlRpcMember("INDEX")]
        [XmlRpcMissingMapping(MappingAction.Ignore)]
        public int Index
        {
            get
            {
                return this.index;
            }

            set
            {
                this.index = value;
            }
        }

        [XmlRpcMember("AES_ACTIVE")]
        [XmlRpcMissingMapping(MappingAction.Ignore)]
        public int IsAesActive
        {
            get
            {
                return this.isAesActive;
            }

            set
            {
                this.isAesActive = value;
            }
        }

        [XmlRpcMember("PARAMSETS")]
        public string[] Paramsets
        {
            get
            {
                return this.paramsets;
            }

            set
            {
                this.paramsets = value;
            }
        }

        [XmlRpcMember("FIRMWARE")]
        [XmlRpcMissingMapping(MappingAction.Ignore)]
        public string FirmwareVersion
        {
            get
            {
                return this.firmwareVersion;
            }

            set
            {
                this.firmwareVersion = value;
            }
        }

        [XmlRpcMember("AVAILABLE_FIRMWARE")]
        [XmlRpcMissingMapping(MappingAction.Ignore)]
        public string AvailableFirmwareVersion
        {
            get
            {
                return this.availableFirmwareVersion;
            }

            set
            {
                this.availableFirmwareVersion = value;
            }
        }

        [XmlRpcMember("VERSION")]
        public int Version
        {
            get
            {
                return this.version;
            }

            set
            {
                this.version = value;
            }
        }

        [XmlRpcMember("FLAGS")]
        public int Flags
        {
            get
            {
                return this.flags;
            }

            set
            {
                this.flags = value;
            }
        }

        [XmlRpcMember("LINK_SOURCE_ROLES")]
        [XmlRpcMissingMapping(MappingAction.Ignore)]
        public string LinkSourceRoles
        {
            get
            {
                return this.linkSourceRoles;
            }

            set
            {
                this.linkSourceRoles = value;
            }
        }

        [XmlRpcMember("LINK_TARGET_ROLES")]
        [XmlRpcMissingMapping(MappingAction.Ignore)]
        public string LinkTargetRoles
        {
            get
            {
                return linkTargetRoles;
            }

            set
            {
                this.linkTargetRoles = value;
            }
        }

        [XmlRpcMember("DIRECTION")]
        [XmlRpcMissingMapping(MappingAction.Ignore)]
        public int Direction
        {
            get
            {
                return this.direction;
            }

            set
            {
                this.direction = value;
            }
        }

        [XmlRpcMember("GROUP")]
        [XmlRpcMissingMapping(MappingAction.Ignore)]
        public string Group
        {
            get
            {
                return this.group;
            }

            set
            {
                this.group = value;
            }
        }

        [XmlRpcMember("TEAM")]
        [XmlRpcMissingMapping(MappingAction.Ignore)]
        public string Team
        {
            get
            {
                return this.team;
            }

            set
            {
                this.team = value;
            }
        }

        [XmlRpcMember("TEAM_TAG")]
        [XmlRpcMissingMapping(MappingAction.Ignore)]
        public string TeamTag
        {
            get
            {
                return this.teamTag;
            }

            set
            {
                this.teamTag = value;
            }
        }

        [XmlRpcMember("TEAM_CHANNELS")]
        [XmlRpcMissingMapping(MappingAction.Ignore)]
        public string[] TeamChannels
        {
            get
            {
                return this.teamChannels;                
            }

            set
            {
                this.teamChannels = value;
            }
        }

        [XmlRpcMember("INTERFACE")]
        [XmlRpcMissingMapping(MappingAction.Ignore)]
        public string InterfaceAddress
        {
            get
            {
                return this.interfaceAddress;
            }

            set
            {
                this.interfaceAddress = value;
            }
        }

        [XmlRpcMember("ROAMING")]
        [XmlRpcMissingMapping(MappingAction.Ignore)]
        public int UseRoaming
        {
            get
            {
                return this.useRoaming;
            }

            set
            {
                this.useRoaming = value;
            }
        }
    }
}
