﻿
namespace Tenma.HomeMatic.Remoting.XmlRpc.Net
{
    using System;
    using System.Collections.Generic;
    using System.Text;

    using CookComputing.XmlRpc;

    public class ParameterDescription : IParameterDescription
    {
        private string parameterType;
        private int operations;
        private int flags;
        private object defaultValue;
        private object minValue;
        private object maxValue;
        private string unit;
        private int tabOrder;
        private string control;
        private Dictionary<string, object> specialValues;
        private String[] valueList;

        public ParameterDescription()
        {
            this.parameterType = string.Empty;
            this.operations = 0;
            this.flags = 0;
            this.defaultValue = null;
            this.minValue = null;
            this.maxValue = null;
            this.unit = string.Empty;
            this.tabOrder = -1;
            this.control = string.Empty;
            this.specialValues = new Dictionary<string, object>(); ;
            this.valueList = new string[] { };
        }

        public string ParameterType
        {
            get
            {
                return this.parameterType;
            }
        }


        public int Operations
        {
            get
            {
                return this.operations;
            }
        }


        public int Flags
        {
            get
            {
                return this.flags;
            }
        }


        public object DefaultValue
        {
            get
            {
                return this.defaultValue;
            }
        }


        public object MinValue
        {
            get
            {
                return this.minValue;
            }
        }


        public object MaxValue
        {
            get
            {
                return this.maxValue;
            }
        }


        public string Unit
        {
            get
            {
                return this.unit;
            }
        }


        public int TabOrder
        {
            get
            {
                return this.tabOrder;
            }
        }


        public string Control
        {
            get
            {
                return this.control;
            }
        }


        public Dictionary<string, object> SpecialValues
        {
            get
            {
                return this.specialValues;
            }
        }


        public String[] ValueList
        {
            get
            {
                return this.valueList;
            }
        }

        public static ParameterDescription Parse(XmlRpcStruct xmlRpcStruct)
        {
            ParameterDescription result = new ParameterDescription();

            result.parameterType = xmlRpcStruct["TYPE"].ToString();
            result.operations = (int)xmlRpcStruct["OPERATIONS"];
            result.flags = (int)xmlRpcStruct["FLAGS"];
            result.defaultValue = xmlRpcStruct["DEFAULT"];
            result.minValue = xmlRpcStruct["MIN"];
            result.maxValue = xmlRpcStruct["MAX"];
            result.unit = xmlRpcStruct["UNIT"].ToString();
            result.tabOrder = (int)xmlRpcStruct["TAB_ORDER"];

            if (xmlRpcStruct.ContainsKey("CONTROL"))
            {
                result.control = xmlRpcStruct["CONTROL"].ToString();
            }

            if (xmlRpcStruct.ContainsKey("SPECIAL_VALUES"))
            {
                XmlRpcStruct specialValues = (XmlRpcStruct)xmlRpcStruct["SPECIAL_VALUES"];
                foreach (String key in specialValues)
                {
                    result.specialValues[key] = specialValues[key];
                }
            }

            if (xmlRpcStruct.ContainsKey("VALUE_LIST"))
            {
                result.valueList = (string[])xmlRpcStruct["VALUE_LIST"];
            }

            return result;
        }

    }
}
