﻿namespace Tenma.HomeMatic.Remoting.XmlRpc.Net
{
    using System;
    using System.Collections;
    using System.Collections.Generic;
    using CookComputing.XmlRpc;
    using Tenma.HomeMatic.Remoting.XmlRpc.Net;

    internal class RpcProxy : IRpcProxy
    {
        private const string DefaultUrl = "http://127.0.0.1:2001/";
        private const int DefaultTimeout = 15 * 1000;
        private IProxy proxy;

        public RpcProxy()
        {
            this.proxy = XmlRpcProxyGen.Create<IProxy>();
            this.proxy.Url = RpcProxy.DefaultUrl;
            this.proxy.Timeout = RpcProxy.DefaultTimeout;
        }

        public string Url
        {
            get
            {
                return this.proxy.Url;
            }

            set
            {
                this.proxy.Url = value;
            }
        }

        public int Timeout
        {
            get
            {
                return this.proxy.Timeout;
            }

            set
            {
                this.proxy.Timeout = value;
            }
        }

        public void ActivateLinkParamset(string address, string peerAddress, bool longPress)
        {
            try
            {
                this.proxy.ActivateLinkParamset(address, peerAddress, longPress);
            }
            catch (Exception ex)
            {
                throw new RpcException(ex);
            }
        }

        public IDeviceDescription AddDevice(string serialNumber)
        {
            try
            {
                return this.proxy.AddDevice(serialNumber);
            }
            catch (Exception ex)
            {
                throw new RpcException(ex);
            }
        }

        public void AddLink(string sender, string receiver, string name, string description)
        {
            try
            {
                this.proxy.AddLink(sender, receiver, name, description);
            }
            catch (Exception ex)
            {
                throw new RpcException(ex);
            }
        }

        public void ChangeKey(string passphrase)
        {
            try
            {
                this.proxy.ChangeKey(passphrase);
            }
            catch (Exception ex)
            {
                throw new RpcException(ex);
            }
        }

        public void ClearConfigCache(string address)
        {
            try
            {
                this.proxy.ClearConfigCache(address);
            }
            catch (Exception ex)
            {
                throw new RpcException(ex);
            }
        }

        public void DeleteDevice(string address, int flags)
        {
            try
            {
                this.proxy.DeleteDevice(address, flags);
            }
            catch (Exception ex)
            {
                throw new RpcException(ex);
            }
        }

        public void DetermineParameter(string address, string paramsetKey, string parameterId)
        {
            try
            {
                this.proxy.DetermineParameter(address, paramsetKey, parameterId);
            }
            catch (Exception ex)
            {
                throw new RpcException(ex);
            }
        }

        public IDeviceDescription GetDeviceDescription(string address)
        {
            try
            {
                return this.proxy.GetDeviceDescription(address);
            }
            catch (Exception ex)
            {
                throw new RpcException(ex);
            }
        }

        public int GetInstallMode()
        {
            try
            {
                return this.proxy.GetInstallMode();
            }
            catch (Exception ex)
            {
                throw new RpcException(ex);
            }
        }

        public string GetKeyMissmatchDevice(bool reset)
        {
            try
            {
                return this.proxy.GetKeyMissmatchDevice(reset);
            }
            catch (Exception ex)
            {
                throw new RpcException(ex);
            }
        }

        public string[] GetLinkInfo(string sender, string receiver)
        {
            try
            {
                return this.proxy.GetLinkInfo(sender, receiver);
            }
            catch (Exception ex)
            {
                throw new RpcException(ex);
            }
        }

        public string[] GetLinkPeers(string address)
        {
            try
            {
                return this.proxy.GetLinkPeers(address);
            }
            catch (Exception ex)
            {
                throw new RpcException(ex);
            }
        }

        public ILinkDescription[] GetLinks(string address, int flags)
        {
            try
            {
                return this.proxy.GetLinks(address, flags);
            }
            catch (Exception ex)
            {
                throw new RpcException(ex);
            }
        }

        public Hashtable GetParamset(string address, string paramsetKey)
        {
            try
            {
                return this.proxy.GetParamset(address, paramsetKey);
            }
            catch (Exception ex)
            {
                throw new RpcException(ex);
            }
        }

        public Dictionary<string, IParameterDescription> GetParamsetDescription(string address, string type)
        {
            try
            {
                Dictionary<string, IParameterDescription> result = new Dictionary<string, IParameterDescription>();

                XmlRpcStruct paramsetDescription = this.proxy.GetParamsetDescription(address, type);
                foreach (String key in paramsetDescription.Keys)
                {
                    result[key] = ParameterDescription.Parse((XmlRpcStruct)paramsetDescription[key]);
                }

                return result;
            }
            catch (Exception ex)
            {
                throw new RpcException(ex);
            }
        }

        public string GetParamsetId(string address, string type)
        {
            try
            {
                return this.proxy.GetParamsetId(address, type);
            }
            catch (Exception ex)
            {
                throw new RpcException(ex);
            }
        }

        public object GetValue(string address, string valueKey)
        {
            try
            {
                return this.proxy.GetValue(address, valueKey);
            }
            catch (Exception ex)
            {
                throw new RpcException(ex);
            }
        }

        public void Init(string url, string interfaceId)
        {
            try
            {
                this.proxy.Init(url, interfaceId);
            }
            catch (Exception ex)
            {
                throw new RpcException(ex);
            }
        }

        public void Shutdown(string url)
        {
            try
            {
                this.proxy.Shutdown(url);
            }
            catch (Exception ex)
            {
                throw new RpcException(ex);
            }
        }

        public IDeviceDescription[] ListDevices()
        {
            try
            {
                return this.proxy.ListDevices();
            }
            catch (Exception ex)
            {
                throw new RpcException(ex);
            }
        }

        public IDeviceDescription[] listTeams()
        {
            try
            {
                return this.proxy.listTeams();
            }
            catch (Exception ex)
            {
                throw new RpcException(ex);
            }
        }

        public int GetLogLevel()
        {
            try
            {
                return this.proxy.GetLogLevel();
            }
            catch (Exception ex)
            {
                throw new RpcException(ex);
            }
        }

        public int SetLogLevel(int logLevel)
        {
            try
            {
                return this.proxy.SetLogLevel(logLevel);
            }
            catch (Exception ex)
            {
                throw new RpcException(ex);
            }
        }

        public void PutParamset(string address, string paramsetKey, Hashtable paramset)
        {
            try
            {
                XmlRpcStruct paramsetStruct = new XmlRpcStruct();
                foreach (String key in paramset.Keys)
                {
                    paramsetStruct[key] = paramset[key];
                }

                this.proxy.PutParamset(address, paramsetKey, paramsetStruct);
            }
            catch (Exception ex)
            {
                throw new RpcException(ex);
            }
        }

        public void RemoveLink(string sender, string receiver)
        {
            try
            {
                this.proxy.RemoveLink(sender, receiver);
            }
            catch (Exception ex)
            {
                throw new RpcException(ex);
            }
        }

        public bool ReportValueUsage(string address, string valueId, int refCounter)
        {
            try
            {
                return this.proxy.ReportValueUsage(address, valueId, refCounter);
            }
            catch (Exception ex)
            {
                throw new RpcException(ex);
            }
        }

        public void RestoreConfigToDevice(string address)
        {
            try
            {
                this.proxy.RestoreConfigToDevice(address);
            }
            catch (Exception ex)
            {
                throw new RpcException(ex);
            }
        }

        public RssiInfo[] GetRssiInfo()
        {
            try
            {
                Dictionary<string, RssiInfo> resultMap = new Dictionary<string, RssiInfo>();

                XmlRpcStruct rssiInfo = this.proxy.GetRssiInfo();
                foreach (String sender in rssiInfo.Keys)
                {
                    XmlRpcStruct rssiValues = (XmlRpcStruct)rssiInfo[sender];
                    foreach (String receiver in rssiValues.Keys)
                    {
                        int[] values = (int[])rssiValues[receiver];

                        if (values[0] != RssiInfo.InvalidRssiValue)
                        {
                            resultMap[sender + "_" + receiver] = new RssiInfo(sender, receiver, values[0]);
                        }

                        if (values[1] != RssiInfo.InvalidRssiValue)
                        {
                            resultMap[receiver + "_" + sender] = new RssiInfo(receiver, sender, values[1]);
                        }
                    }
                }

                RssiInfo[] result = new RssiInfo[resultMap.Count];
                int i = 0;
                foreach (RssiInfo entry in resultMap.Values)
                {
                    result[i] = entry;
                    i++;
                }

                return result;
            }
            catch (Exception ex)
            {
                throw new RpcException(ex);
            }
        }

        public int SearchDevices()
        {
            try
            {
                return this.proxy.SearchDevices();
            }
            catch (Exception ex)
            {
                throw new RpcException(ex);
            }
        }

        public void SetInstallMode(bool isOn)
        {
            try
            {
                this.proxy.SetInstallMode(isOn);
            }
            catch (Exception ex)
            {
                throw new RpcException(ex);
            }
        }

        public void SetLinkInfo(string sender, string receiver, string name, string description)
        {
            try
            {
                this.proxy.SetLinkInfo(sender, receiver, name, description);
            }
            catch (Exception ex)
            {
                throw new RpcException(ex);
            }
        }

        public void SetTeam(string channelAddress, string teamAddress)
        {
            try
            {
            }
            catch (Exception ex)
            {
                throw new RpcException(ex);
            }
            this.proxy.SetTeam(channelAddress, teamAddress);
        }

        public void SetTempKey(string passphrase)
        {
            try
            {
                this.proxy.SetTempKey(passphrase);
            }
            catch (Exception ex)
            {
                throw new RpcException(ex);
            }
        }

        public void SetValue(string address, string valueKey, object value)
        {
            try
            {
                this.proxy.SetValue(address, valueKey, value);
            }
            catch (Exception ex)
            {
                throw new RpcException(ex);
            }
        }

        public bool[] UpdateFirmware(string[] deviceAddresses)
        {
            try
            {
                return this.proxy.UpdateFirmware(deviceAddresses);
            }
            catch (Exception ex)
            {
                throw new RpcException(ex);
            }
        }

        public IBidcosInterface[] ListBidcosInterfaces()
        {
            try
            {
                return this.proxy.ListBidcosInterfaces();
            }
            catch (Exception ex)
            {
                throw new RpcException(ex);
            }
        }

        public void SetBidCosIntrface(string deviceAddress, string interfaceAddress, bool useRoaming)
        {
            try
            {
                this.proxy.SetBidCosIntrface(deviceAddress, interfaceAddress, useRoaming);
            }
            catch (Exception ex)
            {
                throw new RpcException(ex);
            }
        }

        public IServiceMessage[] GetServiceMessages()
        {
            try
            {
                object[] obj = this.proxy.GetServiceMessages();
                ServiceMessage[] result = new ServiceMessage[obj.Length];

                for (int i = 0; i < obj.Length; i++)
                {
                    result[i] = ServiceMessage.Parse(obj[i]);
                }

                return result;
            }
            catch (Exception ex)
            {
                throw new RpcException(ex);
            }
        }

        public object GetMetadata(string objectId, string dataId)
        {
            try
            {
                return this.proxy.GetMetadata(objectId, dataId);
            }
            catch (Exception ex)
            {
                throw new RpcException(ex);
            }
        }

        public void SetMetadata(string objectId, string dataId, object value)
        {
            try
            {
                this.proxy.SetMetadata(objectId, dataId, value);
            }
            catch (Exception ex)
            {
                throw new RpcException(ex);
            }
        }

        public Hashtable GetAllMetadata(string objectId)
        {
            try
            {
                return this.proxy.GetAllMetadata(objectId);
            }
            catch (Exception ex)
            {
                throw new RpcException(ex);
            }
        }
    }
}
