﻿namespace Tenma.HomeMatic.Remoting.XmlRpc.Net
{
    /// <summary>
    /// Kapselt eine Service-Meldung.
    /// </summary>
    public class ServiceMessage : IServiceMessage
    {
        /// <summary>
        /// Seriennummer der Quelle.
        /// </summary>
        private string address;

        /// <summary>
        /// Id der Service-Meldung.
        /// </summary>
        private string id;

        /// <summary>
        /// Wert der Service-Meldung.
        /// </summary>
        private object value;

        /// <summary>
        /// Erstellt und initialisiert eine
        /// neue Service-Meldung.
        /// </summary>
        public ServiceMessage()
        {
            this.address = string.Empty;
            this.id = string.Empty;
            this.value = null;
        }

        /// <summary>
        /// Liefert die Seriennummer der Quelle.
        /// </summary>
        public string Address
        {
            get
            {
                return this.address;
            }
        }

        /// <summary>
        /// Liefert die Id der Seriennummer.
        /// </summary>
        public string Id
        {
            get
            {
                return this.id;
            }
        }

        /// <summary>
        /// Liefert den Wert der Servicemeldung.
        /// </summary>
        public object Value
        {
            get
            {
                return this.value;
            }
        }

        /// <summary>
        /// Parst eine Servicemeldung.
        /// </summary>
        /// <param name="value">Zu parsende Objekt</param>
        /// <returns>Geparste Servicemeldung</returns>
        public static ServiceMessage Parse(object value)
        {
            ServiceMessage result = new ServiceMessage();
            
            object[] list = (object[])value;
            result.address = list[0].ToString();
            result.id = list[1].ToString();
            result.value = list[2];

            return result;
        }
    }
}
